/**

OtherPairs

From a given currency pair, determine the base currency, then determine how many of
the "other" pairs having that base currency have 20EMA above 50EMA.

In cases that the base currency is actually the quote currency of the test pair,
reverse the order, i.e. 20EMA below 50EMA.

For example: USDCAD has base currency USD.
* If USDCAD has 20EMA above 50EMA, that's +1.
* If EURUSD has 20EMA below 50EMA, that's also +1, because USD is the quote currency.

*/

#property copyright		"2018 Anthony J. Garot"
#property link			"http://www.garot.com/trading/"
#property version		"1.14"
#property icon			"\\Images\\OtherPairs.ico"
#property description	"From a given currency pair, determine the base currency, then determine how many of the pairs have 20EMA above 50EMA."
#property description	" "
#property description	"MQL5 Profile: https://www.mql5.com/en/users/tonegarot"
#property description	"My Website: http://www.garot.com/trading/"
#property strict

//--- indicator settings
#property indicator_separate_window
//#property indicator_chart_window

#property indicator_minimum		-7.02				// I have this 1.02 + the maximum it can be
#property indicator_maximum		+7.02

#property indicator_height		175

#property indicator_buffers 	2
#property indicator_plots		1

#property indicator_type1		DRAW_COLOR_HISTOGRAM
#property indicator_color1		CLR_NONE,SeaGreen,FireBrick,LightGreen,Orange
#property indicator_width1		2

#define ALERT(x) Print(x)
#define THRESHOLD				4.			// Major trend threshold
#define DEFAULT_MAX_BARS		509			// a good default number for D1
#define ABSOLUTE_MAX_BARS		3000		// based upon CADJPY, which has 3035 as of 2018.11.07
#define HR2400					86400			 	// 24 * 60 * 60
#define MIDNIGHT(x) x - (int) MathMod(x,HR2400)

#include <ChartObjects\ChartObjectsTxtControls.mqh>

struct SPair
{
	string	symbol;
	int		direction;			// +1 or -1
	// handles
	int		FastHandle;
	int		MediumHandle;
	// MA's
	double	FastBuffer[];		// internal buffer
	double	MediumBuffer[];		// internal buffer

	SPair()
	{
		ArraySetAsSeries(FastBuffer, true);				// Allows max bars construct
		ArraySetAsSeries(MediumBuffer, true);			// Allows max bars construct
	}
};

// Note: this matches the order of the majors[] array above.
enum EnMajors
{
	MAJOR_EURUSD,	// EURUSD
	MAJOR_USDJPY,	// USDJPY
	MAJOR_GBPUSD,	// GBPUSD
	MAJOR_USDCAD,	// USDCAD
	MAJOR_USDCHF,	// USDCHF
	MAJOR_AUDUSD,	// AUDUSD
	MAJOR_NZDUSD	// NZDUSD
};

class CIndicatorComment
{
public:
	// vars
	CChartObjectLabel	m_label;
	int					m_subWindow;

	// Constructor
	CIndicatorComment()	{}

	// Destructor
	~CIndicatorComment()
	{
		m_label.Delete();
	}

	void Init(const int sub_window)
	{
		m_subWindow = sub_window;

		pCreate();
	}

	// Straight through
	void Description(const string message)
	{
		if ( m_label.CreateTime() == 0 ) return;

		m_label.Description(message);
		ChartRedraw();
	}

	// Updates the comment and dumps a line to the log file.
	// This could be extended in future to display something on the chart.
	void DescriptionSuppression(const string message)
	{
		static string message_hold = "";

		if ( m_label.CreateTime() == 0 ) return;

		// One thing to keep in mind is that the Loading message only
		// shows once if you get into an infinite loop.
		if ( message != message_hold )
		{
			//Comment(message);						// This has an auto ChartRedraw() built-in
			if ( DEBUG ) Print(message);
			m_label.Description(message);			// Text to display
			ChartRedraw();							// I got this to work in OnTimer(), then it didn't work. Huh.
			message_hold = message;
		}
	}

	bool IsQuiet()
	{
		datetime createTime = m_label.CreateTime();
		bool isQuiet = (createTime == 0 );
		PrintFormat("%s(): createTime [%d] IsQuiet [%d]", __FUNCTION__, createTime, isQuiet);
		return isQuiet;
	}
	void Active() { Print("Indicator Comment going active"); pCreate(); }
	void GoQuiet() { Print("Indicator Comment going quiet"); pDelete(); }

private:
	void pCreate()
	{
		if ( m_subWindow == NULL )
		{
			ALERT("m_subWindow == NULL . . . cannot continue");
		}
		/**
		 * https://www.mql5.com/en/docs/standardlibrary/chart_object_classes/obj_controls/cchartobjectlabel/cchartobjectlabelcreate
		 * Interesting that Create() will remove the object if the same name is found.
		 *
		 * Limited to 63 characters
		*/
		string name = "loading";
		if(m_label.Create(chartID,name,m_subWindow,10,25))
		{
			m_label.Anchor(ANCHOR_LEFT);
			//m_label.Angle(90);				// Rotate to side
			m_label.Description("");			// Text to display
			m_label.Color(clrDarkOrange);
			m_label.Font("Consolas");
			m_label.FontSize(9);
			//m_label.Detach();				// Detaches the graphical object.
		}
		ChartRedraw(chartID);
	}

	void pDelete()
	{
		m_label.Delete();
		ChartRedraw();
	}
};

//---- constants
const int 					MAIN_WINDOW = 0;
const int 					SUB_WINDOW = 1;
const int 					chartID = 0;
const int					START_POS = 0;
const double				COLOR_NONE = 0.;
const double				COLOR_POSITIVE = 1.;
const double				COLOR_NEGATIVE = 2.;
const double				COLOR_CLEAR_MAJORITY = 3.;
const double				COLOR_CLEAR_MINORITY = 4.;
const ENUM_APPLIED_PRICE	APPLIED_PRICE=PRICE_CLOSE;	// Applied Price
const int					NUM_SYMBOLS = 7;
const string majors[] = {
	"EURUSD",					// 7 of 7 share EUR as base currency
	"USDJPY",					// 3 of 7 share USD as base currency
	"GBPUSD",					// 6 of 7 share GBP as base currency
	"USDCAD",
	"USDCHF",
	"AUDUSD",					// 5 of 7 share AUD as base currency
	"NZDUSD"					// 4 of 7 share NZD as base currency
};
// This is all the possible combinations of the 6 major currencies.
// There are 28 of them. It doesn't consider PLN, SGD, NOK, SEK, DKK, etc.,
// even though those are in the minors.
//
// EUR / USD
//		EUR == base currency
//		USD == quote currency
const string currencyPairs[] = {
	"EURUSD",		// major
	"GBPUSD",		// major
	"USDJPY",		// major
	"EURGBP",		// major
	"EURJPY",		// major
	"GBPJPY",		// major
	"USDCAD",		// major
	"CADCHF",		// major
	"USDCHF",		// major
	"CHFJPY",		// major
	"EURCHF",		// major
	"GBPCHF",		// major
	"CADJPY",		// major
	"EURCAD",		// major
	"GBPCAD",		// major
	"AUDUSD",		//	minor
	"NZDUSD",		//	minor
	"AUDCHF",		//	minor
	"NZDCHF",		//	minor
	"AUDCAD",		//	minor
	"AUDJPY",		//	minor
	"AUDNZD",		//	minor
	"EURAUD",		//	minor
	"EURNZD",		//	minor
	"GBPAUD",		//	minor
	"GBPNZD",		//	minor
	"NZDCAD",		//	minor
	"NZDJPY"		//	minor
};

//---- input parameters
input string			InpBaseCurrency="" /* base = USD; quote = CAD */;	// Base currency (Blank to use chart)
input int				InpMaFastPeriod=20;				// Fast MA period
input int				InpMaMediumPeriod=50;			// Medium MA period
input ENUM_MA_METHOD	InpMaFastMethod=MODE_EMA;		// Fast MA Method
input ENUM_MA_METHOD	InpMaMediumMethod=MODE_EMA;		// Medium MA Method
input int				InpMaxBars=DEFAULT_MAX_BARS;	// Num bars to show/download
input bool				InpShowIndicatorComment = true;	// Show Indicator Comment (OBJ_LABEL)

//---- indicator buffers
// external buffers
double	ExtHistoBuffer[];
double	ExtHistoColors[];


//---- globals
CIndicatorComment	*g_indicatorComment = NULL;
ENUM_TIMEFRAMES		g_chartTimeFrame = PERIOD_D1;
int					g_barsMinimum = -1;			// Minimum number of bars to perform a calculation?
int					g_maxBars;
datetime			g_maxBarTime = NULL;		// Related to g_maxBars
SPair				pairs[7];					// Other pairs (NUM_SYMBOLS)
int					g_debugBar = 0;				// set to an index to turn DEBUG on for that bar
datetime			g_debugTime = 0;			// e.g. D'2018.11.06' set to specific time to turn DEBUG on for that bar
int					DEBUG = 0 /* The greater the number, the more verbose. */;	// DEBUG 0=off;1=verbose;2=spray;3=diarrhea


/**
 * Per whroeder1
 * Don't try to use any price or server related functions in OnInit (or on load,)
 * as there may be no connection/chart yet
 */
int OnInit()
{
	if ( DEBUG )
	{
		PrintFormat("%s(): symbol [%s] max bars [%d]", __FUNCTION__, Symbol(), TerminalInfoInteger(TERMINAL_MAXBARS) );
		PrintFormat("%s(): MQL_PROGRAM_TYPE [%d]",	__FUNCTION__, MQLInfoInteger(MQL_PROGRAM_TYPE));
		PrintFormat("%s(): MQL_PROGRAM_NAME [%s]",	__FUNCTION__, MQLInfoString(MQL_PROGRAM_NAME));
		PrintFormat("%s(): MQL5_PROGRAM_PATH [%s]",	__FUNCTION__, MQLInfoString(MQL_PROGRAM_PATH));
		//return INIT_FAILED;
	}

	// Having a large # of bars for rates_total means pulling a lot of data
	// for other currencies. This has been alleviated with InpMaxBars to an
	// extent; however, it's good to limit the bars in the chart first.
	//
	// I created a .ini file to start MT5 where this can be set.
	if ( TerminalInfoInteger(TERMINAL_MAXBARS) > 5000 )
	{
		PrintFormat("Warning! TERMINAL_MAXBARS is [%d]. To minimize download of data for other pairs, keep this small.", TerminalInfoInteger(TERMINAL_MAXBARS) );
	}

	g_maxBars = InpMaxBars;
	if ( g_maxBars == 0 ) g_maxBars = DEFAULT_MAX_BARS;			// Can't have 0 bars
	g_maxBars = MathMin(g_maxBars,TerminalInfoInteger(TERMINAL_MAXBARS));
	g_maxBars = MathMin(g_maxBars,ABSOLUTE_MAX_BARS);			// never greater than ABSOLUTE_MAX_BARS
	if ( DEBUG ) PrintFormat("%s(): g_maxBars [%d]", __FUNCTION__, g_maxBars );

	Comment("");

	string shortName = StringFormat("TommorOtherPairs (%d,%d)", InpMaFastPeriod, InpMaMediumPeriod);
	if ( DEBUG ) PrintFormat("%s(): shortName %s", __FUNCTION__, shortName);

	string baseCurrency;
	switch(StringLen(InpBaseCurrency))
	{
		case 0:
			baseCurrency = StringSubstr(Symbol(),0,3);		// Use the chart currency pair
			break;

		case 3:
			baseCurrency = InpBaseCurrency;
			break;

		case 6:
			baseCurrency = StringSubstr(InpBaseCurrency,0,3);
			break;

		default:
			ALERT(StringFormat("Unknown base currency [%s]", InpBaseCurrency));
			return INIT_FAILED;
	}
	if ( DEBUG ) PrintFormat("%s(): baseCurrency [%s]", __FUNCTION__, baseCurrency);
	if ( InpShowIndicatorComment ) Comment(StringFormat("Base currency [%s] Max Bars [%d] Initial Count [%d]", baseCurrency, g_maxBars, Bars(NULL,g_chartTimeFrame)) );

	// bars minimum for calculation
	g_barsMinimum = InpMaFastPeriod;
	g_barsMinimum = MathMax(g_barsMinimum, InpMaMediumPeriod);
	if ( DEBUG ) PrintFormat("%s(): g_barsMinimum [%d]", __FUNCTION__, g_barsMinimum);

	ArraySetAsSeries(ExtHistoBuffer, true);
	ArraySetAsSeries(ExtHistoColors, true);

	//---- indicator buffers mapping
	SetIndexBuffer(0,ExtHistoBuffer,INDICATOR_DATA);
	SetIndexBuffer(1,ExtHistoColors,INDICATOR_COLOR_INDEX);

	PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,g_barsMinimum);
	PlotIndexSetString(0,PLOT_LABEL,"HISTO");
	PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,0);			// Set as an empty value 0
	PlotIndexSetString(1,PLOT_LABEL,"");

	IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
	IndicatorSetString(INDICATOR_SHORTNAME,shortName);

	if ( ! pGenerateRelevantPairsFromBaseCurrency(baseCurrency) )
	{
		PrintFormat("%s(): pGenerateRelevantPairsFromBaseCurrency failed", __FUNCTION__);
		return (INIT_FAILED);
	}

	// Start a load for D1. The first load is usually 510.
	// Start a load of data to ensure Tester has access to this currency.
	// Per Alain Verleyen:
	// https://www.mql5.com/en/forum/286608#comment_9184731
	for ( int j=0; j<NUM_SYMBOLS;j++ )
	{
		if(IsStopped()) return (INIT_FAILED); //Checking for stop flag

		datetime start_time = MIDNIGHT(TimeCurrent());
		datetime times[];
		ResetLastError();
		int countTime=CopyTime(pairs[j].symbol,g_chartTimeFrame,start_time,g_maxBars,times);
		if ( DEBUG ) PrintFormat("%s(): Initial Load of [%s]. CopyTime() returned [%d]", __FUNCTION__, pairs[j].symbol, countTime);
		if ( countTime == -1 )
		{
			// This is NOT necessarily an error.
			if ( DEBUG ) PrintFormat("%s(): count [-1] symbol [%s] TF [%s] start_time [%s]. Error [%d]",
				__FUNCTION__,
				pairs[j].symbol,
				EnumToString(g_chartTimeFrame),
				TimeToString(start_time),
				GetLastError());
		}
	}

	g_indicatorComment = new CIndicatorComment();
	if ( InpShowIndicatorComment && ! IsBackTester() )
	{
		g_indicatorComment.Init(SUB_WINDOW);
	}

	// initialization done
	if ( DEBUG ) PrintFormat("%s(): Initialization succeeded", __FUNCTION__);
	return(INIT_SUCCEEDED);
}

// Reasons: https://www.mql5.com/en/docs/constants/namedconstants/uninit
void OnDeinit(const int reason)
{
	if ( DEBUG ) PrintFormat("%s(): reason [%d]", __FUNCTION__, reason);

	delete(g_indicatorComment);
}

void OnChartEvent(const int id,         // Event identifier
                  const long& lparam,   // Event parameter of long type
                  const double& dparam, // Event parameter of double type
                  const string& sparam  // Event parameter of string type
                  )
{
	if ( id == CHARTEVENT_CLICK )
	{
		if ( DEBUG ) PrintFormat("CHARTEVENT_CLICK id [%d] lparam [%d] dparam [%f] sparam [%s]", id, lparam, dparam, sparam);

		if ( g_indicatorComment.IsQuiet() )
		{
			g_indicatorComment.Active();
		}
		else
		{
			g_indicatorComment.GoQuiet();
		}
	}

	// Click an OBJ_EDIT, for example.
	// NOTE: CHARTEVENT_CLICK fires right after this, every time.
	if ( id == CHARTEVENT_OBJECT_CLICK )
	{
		if ( DEBUG ) PrintFormat("CHARTEVENT_OBJECT_CLICK id [%d] lparam [%d] dparam [%f] sparam [%s]", id, lparam, dparam, sparam);

		string name = sparam;		// Object name
		string text = ObjectGetString(chartID,name,OBJPROP_TEXT);

		if ( DEBUG ) PrintFormat("Clicked on [%s] [%s]", name, text);

		//if ( name == "loading" ) {}
	}
}

// Use the long form so as not to get the Parameters tab
int OnCalculate (const int rates_total,      // size of input time series
                 const int prev_calculated,  // bars handled in previous call
                 const datetime& time[],     // Time
                 const double& open[],       // Open
                 const double& high[],       // High
                 const double& low[],        // Low
                 const double& close[],      // Close
                 const long& tick_volume[],  // Tick Volume
                 const long& volume[],       // Real Volume
                 const int& spread[]         // Spread
   )
{
	// This gets noisy
	if ( DEBUG > 1 ) PrintFormat("%s(): Entered. rates_total [%d] prev_calculated [%d] earliest time [%s] g_barsMinimum [%d]",
		__FUNCTION__,
		rates_total,
		prev_calculated,
		TimeToString(time[0]),
		g_barsMinimum);

	// Do all functions you cannot do in OnInit() here.
	if ( prev_calculated == 0 )
	{
		ArrayInitialize(ExtHistoBuffer,0.);
		ArrayInitialize(ExtHistoColors,COLOR_NONE);
	}

	// How many bars
	int numBars = rates_total;								// Start with # of rates, e.g. 5000
	if ( numBars > g_maxBars ) { numBars = g_maxBars; }		// Whittle down to usable amount, e.g. 2000

	int to_copy = numBars;
	int begin = numBars - 1;								// starting point

	// No need to pull everything over again.
	// After the initial pull, only need the value of the first filled bar,
	// which is bar 1. The 0th bar is the instantaneous value, which is garbage.
	if ( prev_calculated > 0 )
	{
		begin = rates_total - prev_calculated;		// usually 1
		to_copy = begin + 1;						// should be 2
	}

	// Ensure we have enough data for all other currency pairs.
	if ( ! pEnsureData(rates_total, time[0]) )
	{
		if ( DEBUG ) PrintFormat("pEnsureData . . . returning rates_total [%d] which will become prev_calculated", rates_total);
		return 0;
	}
	else
	{
		if ( ! pFillBuffers(START_POS, to_copy) )
		{
			if ( DEBUG ) PrintFormat("pFillBuffers failed. rates_total [%d] to_copy [%d] start_pos [%d]", rates_total, to_copy, START_POS);
			return 0;
		}
	}

	if ( DEBUG && prev_calculated != rates_total ) PrintFormat("prev_calculated [%d] rates_total [%d] numBars [%d] begin [%d]",
		prev_calculated,
		rates_total,
		numBars,
		begin);

	//--- the main loop of calculations
	// Note that I don't want the 0th bar because it hasn't been filled yet.
	for(int i=begin;i>0 && !IsStopped();i--)
	{
		// Start with nuthin'
		ExtHistoBuffer[i] = 0.;

		if ( numBars - i < g_barsMinimum )
		{
			if ( DEBUG && numBars - i > 1 ) PrintFormat("Skipping [%d] because less than g_barsMinimum [%d]", i, g_barsMinimum);

			ExtHistoBuffer[i] = 0.;
			ExtHistoColors[i] = COLOR_NONE;
			continue;
		}

		if ( g_debugBar > 0 )
		{
			DEBUG=0;
			if ( i == g_debugBar )
			{
				DEBUG=4;
				// Using ArraySetAsSeries(), must find time from left end.
				PrintFormat("g_debugBar [%d] time [%s]", g_debugBar, TimeToString(time[rates_total - g_debugBar - 1]) );
			}
		}
		if ( g_debugTime > 0 )
		{
			DEBUG=0;
			if ( time[rates_total - 1 - i] == g_debugTime )
			{
				DEBUG=2;
				PrintFormat("g_debugTime [%s] bar [%d] i [%d]", TimeToString(g_debugTime), rates_total - 1 - i, i );
			}
		}

		// 11. Does a clear majority of other pairs with same base currency
		// (first currency of pair) also show 20EMA above 50EMA?
		int longΣ = 0;
		int shortΣ = 0;
		for ( int j=0; j<NUM_SYMBOLS;j++ )
		{
			if ( DEBUG && prev_calculated > 0 ) PrintFormat("i [%d] j [%d] ArraySize(Fast) [%d] ArraySize(Slow) [%d]",
															i,
															j,
															ArraySize(pairs[j].FastBuffer),
															ArraySize(pairs[j].MediumBuffer)
															);

			string aboveBelow = "";
			if ( pairs[j].FastBuffer[i] > pairs[j].MediumBuffer[i] )
			{
				aboveBelow = "Above";
				if ( pairs[j].direction == 1 )
				{
					longΣ++;
				}
				else
				{
					shortΣ++;
				}
			}
			else if ( pairs[j].FastBuffer[i] < pairs[j].MediumBuffer[i]  )
			{
				aboveBelow = "Below";
				if ( pairs[j].direction == -1 )
				{
					longΣ++;
				}
				else
				{
					shortΣ++;
				}
			}
			else
			{
				aboveBelow = "ERROR";
				PrintFormat("ERROR: Can't tell if above or below. symbol [%s] Fast [%f] Medium [%f]", pairs[j].symbol, pairs[j].FastBuffer[i], pairs[j].MediumBuffer[i]);
			}

			if ( DEBUG > 1 ) PrintFormat("Fast %s D%+d: i [%d](%d) [%s] [%s] Fast [%.5f] Slow [%.5f] longΣ [%d] shortΣ [%d]",
				aboveBelow,
				pairs[j].direction,
				i,
				rates_total - 1 - i,
				TimeToString(time[rates_total - 1 - i]),
				pairs[j].symbol,
				pairs[j].FastBuffer[i],
				pairs[j].MediumBuffer[i],
				longΣ,
				shortΣ );
		}
		if ( DEBUG > 3 ) PrintFormat("longΣ [%d] shortΣ [%d] Sum [%d] Diff [%d]", longΣ, shortΣ, longΣ + shortΣ, longΣ - shortΣ);

		ExtHistoColors[i] = COLOR_NONE;
		if ( longΣ > shortΣ )
		{
			ExtHistoBuffer[i] = longΣ;
			ExtHistoColors[i] = COLOR_POSITIVE;
			if ( ExtHistoBuffer[i] > THRESHOLD ) ExtHistoColors[i] = COLOR_CLEAR_MAJORITY;
		}
		else
		{
			ExtHistoBuffer[i] = -shortΣ;
			ExtHistoColors[i] = COLOR_NEGATIVE;
			if ( ExtHistoBuffer[i] < -THRESHOLD ) ExtHistoColors[i] = COLOR_CLEAR_MINORITY;
		}
	}

	// return value of prev_calculated for next call
	if ( DEBUG > 1 ) PrintFormat("Returning rates_total [%d]", rates_total);
	return(rates_total);
}

/**
 * Remember: Do this in OnCalculate(), not in OnInit(). See note there for more info.
 */
bool pEnsureData(int rates_total, datetime start_time)
{
	if ( DEBUG > 1 ) PrintFormat("%s(): Entered. rates_total [%d] start_time [%s] g_maxBarTime [%d]",
								__FUNCTION__,
								rates_total,
								TimeToString(start_time),
								TimeToString(g_maxBarTime) );

	if ( start_time < g_maxBarTime ) { start_time = g_maxBarTime; }
	if ( rates_total > g_maxBars ) { rates_total = g_maxBars; }

	bool isSynchronized = false;
	while ( ! isSynchronized && ! IsStopped() )
	{
		isSynchronized = true;		// until proven guilty

		// Loop over all symbols
		for ( int j=0; j<NUM_SYMBOLS;j++ )
		{
			if(IsStopped()) return false; //Checking for stop flag

			// not all data may be calculated
			int calculated;
			ResetLastError();
			calculated=BarsCalculated(pairs[j].FastHandle);

			// sometimes happens for some reason
			if ( calculated == 0  )
			{
				if ( DEBUG ) PrintFormat("[%s] 0 calculated for some reason", pairs[j].symbol);
				isSynchronized = false;
				//break;
			}

			if(calculated<rates_total)
			{
				// Need more calculated

				string loading = StringFormat("Loading data for MA(%d) [%s] (%d bars < %d needed)",
					InpMaFastPeriod,
					pairs[j].symbol,
					calculated,
					rates_total );
				g_indicatorComment.DescriptionSuppression(loading);

				if ( pairs[j].symbol != Symbol() )
				{
					datetime singleTime = WrapCopyTimeSingle(start_time, pairs[j].symbol, 0);
					if ( DEBUG > 2 ) PrintFormat("%s(): singleTime [%s]", __FUNCTION__, TimeToString(singleTime) );
					if ( singleTime == NULL )
					{
						isSynchronized = false;
						break;
					}
				}
			}
			else
			{
				if ( DEBUG > 2 ) PrintFormat(
					"BarsCalculated() OK. (%d of %d needed) for symbol [%s]",
					calculated,
					rates_total,
					pairs[j].symbol );
			}

			ResetLastError();
			calculated=BarsCalculated(pairs[j].MediumHandle);

			// sometimes happens for some reason
			if ( calculated == 0 )
			{
				if ( DEBUG ) PrintFormat("[%s] 0 calculated for some reason", pairs[j].symbol);
				isSynchronized = false;
				break;
			}

			if(calculated<rates_total)
			{
				// Need more calculated

				string loading = StringFormat("Loading data for MA(%d) [%s] (%d bars < %d needed)",
					InpMaMediumPeriod,
					pairs[j].symbol,
					calculated,
					rates_total );
				g_indicatorComment.DescriptionSuppression(loading);

				if ( pairs[j].symbol != Symbol() )
				{
					datetime singleTime = WrapCopyTimeSingle(start_time, pairs[j].symbol, 0);
					if ( DEBUG ) PrintFormat("singleTime [%s]", TimeToString(singleTime) );
					if ( singleTime == NULL )
					{
						isSynchronized = false;
						break;
					}
				}
			}
			else
			{
				if ( DEBUG > 2 ) PrintFormat(
					"BarsCalculated() OK. (%d of %d needed) for symbol [%s]",
					calculated,
					rates_total,
					pairs[j].symbol );
			}
		}
	}

	if ( DEBUG > 1 ) Print("Synchronized!");
	g_indicatorComment.Description( StringFormat("Synchronized to %s!", TimeToString(start_time, TIME_DATE)) );			// Text to display
	return true;
}

bool pFillBuffers(const int start_pos, int to_copy)
{
	if ( DEBUG > 1 && to_copy > 1 ) PrintFormat("%s(): start_pos [%d] to_copy [%d]", __FUNCTION__, start_pos, to_copy);

	if ( to_copy > g_maxBars ) { to_copy = g_maxBars; }

	bool gotIssue = false;
	for ( int j=0; j<NUM_SYMBOLS;j++ )
	{
		if(IsStopped()) return false; //Checking for stop flag

		int copied;
		ResetLastError();
		copied = CopyBuffer(pairs[j].FastHandle,0,start_pos,to_copy,pairs[j].FastBuffer);		// from iMA, buffer 0
		if( copied < to_copy )
		{
			if ( DEBUG ) PrintFormat("CopyBuffer() loading for iMA(%d) [%s] (%d<%d) Code [%d].", InpMaFastPeriod, pairs[j].symbol, copied, to_copy, GetLastError());
			gotIssue = true;
		}
		if ( DEBUG > 2 && copied > 1 ) PrintFormat("CopyBuffer() OK for iMA(%d) [%s] returned [%d] ArraySize [%d].", InpMaFastPeriod, pairs[j].symbol, copied, ArraySize(pairs[j].FastBuffer) );

		ResetLastError();
		copied = CopyBuffer(pairs[j].MediumHandle,0,start_pos,to_copy,pairs[j].MediumBuffer);	// from iMA, buffer 0
		if( copied < to_copy )
		{
			if ( DEBUG ) PrintFormat("CopyBuffer() loading for iMA(%d) [%s] (%d<%d) Code [%d].", InpMaMediumPeriod, pairs[j].symbol, copied, to_copy, GetLastError());
			gotIssue = true;
		}
		if ( DEBUG > 2 && copied > 1 ) PrintFormat("CopyBuffer() OK for iMA(%d) [%s] returned [%d] ArraySize [%d].", InpMaMediumPeriod, pairs[j].symbol, copied, ArraySize(pairs[j].MediumBuffer) );
	}
	if ( gotIssue ) return false;

	if ( DEBUG > 1 ) Print("Buffers Filled!");
	return true;
}

// Load the 7 symbols for the base currency into the pairs[] array,
// Then set up the fast and medium iMA's for each currency pair.
bool pGenerateRelevantPairsFromBaseCurrency(const string baseCurrency)
{
	int j = 0;
	for(int i = 0; i<ArraySize(currencyPairs);i++)
	{
		bool gotPair = false;			// guilty until proven innocent

		string base = StringSubstr(currencyPairs[i],0,3);
		string quote = StringSubstr(currencyPairs[i],3,3);
		if ( base == baseCurrency )
		{
			pairs[j].symbol = currencyPairs[i];
			pairs[j].direction = 1;
			if ( DEBUG > 1 ) PrintFormat("[%d] Symbol [%s] is included [%d]", j, pairs[j].symbol, pairs[j].direction);
			gotPair = true;
		} else if ( quote == baseCurrency )
		{
			pairs[j].symbol = currencyPairs[i];
			pairs[j].direction = -1;
			if ( DEBUG > 1 ) PrintFormat("[%d] Symbol [%s] is included [%d]", j, pairs[j].symbol, pairs[j].direction);
			gotPair = true;
		}

		if ( gotPair )
		{
			// get handles for the indicators used in this indicator
			pairs[j].FastHandle=iMA(
				pairs[j].symbol,	// symbol
				g_chartTimeFrame,	// period
				InpMaFastPeriod,	// ma_period
				0,					// ma_shift
				InpMaFastMethod,	// ma_method
				APPLIED_PRICE		// applied_price
				);
			if ( pairs[j].FastHandle == INVALID_HANDLE )
			{
				ALERT(StringFormat("Error creating indicator handle for [%s] InpMaFastPeriod [%d]",
									pairs[j].symbol,
									InpMaFastPeriod));
				return false;
			}

			pairs[j].MediumHandle=iMA(
				pairs[j].symbol,	// symbol
				g_chartTimeFrame,	// period
				InpMaMediumPeriod,	// ma_period
				0,					// ma_shift
				InpMaMediumMethod,	// ma_method
				APPLIED_PRICE		// applied_price
				);
			if ( pairs[j].MediumHandle == INVALID_HANDLE )
			{
				ALERT(StringFormat("Error creating indicator handle for [%s] InpMaMediumPeriod [%d]",
									pairs[j].symbol,
									InpMaMediumPeriod));
				return false;
			}

			j++;
		}
	}

	return true;
}

/**
Attempt to copy the time.
https://www.mql5.com/en/docs/series/copytime
Into an array created for this purpose array (tmpTimeArray), copy the opening
of the zero bar of each affected instrument. CopyTime will cause a pull of more data.
ERR_HISTORY_NOT_FOUND	4401	Requested history not found

For Weekly, CopyTime() will always return the Sunday value.
*/
datetime WrapCopyTimeSingle(const datetime bartime, const string symbol = NULL, const ENUM_TIMEFRAMES tf = 0)
{
	datetime tmpTimeArray[1];	// Intermediate array for the time of the bar.

	ResetLastError();
	int copied = CopyTime(symbol,tf,bartime,1,tmpTimeArray);	// Requesting only 1 bar!

	if ( copied == -1 )
	{
		if ( DEBUG > 2 )
		{
			// This is NOT an error. This is expected behavior while data
			// is pulled from the broker. Even so, I made it DEBUG level 1.
			PrintFormat("%s(): count [-1] symbol [%s] TF [%s] bartime [%s]. Error [%d]",
						__FUNCTION__,
						symbol,
						EnumToString(tf),
						TimeToString(bartime),
						GetLastError());
		}

		// This is OK. I'll just let OnCalculate() know, and it will run that bar again.
		return NULL;
	}

	if ( DEBUG > 2 )
	{
		PrintFormat("%s(): [%s] copied count [%d] TF [%s] value [%s]",
					__FUNCTION__,
					TimeToString(bartime),
					copied,
					EnumToString(tf),
					TimeToString(tmpTimeArray[0]));
	}

	return tmpTimeArray[0];
}

bool IsBackTester(void)
{
	return MQLInfoInteger(MQL_OPTIMIZATION) || MQLInfoInteger(MQL_TESTER);
}

// Figure out the time at which downloading should stop.
// For CADJPY, as of 2018.11.07, this was 3035.
int GetEarliestBar(const string symbol, const ENUM_TIMEFRAMES period, const datetime earliestTime, const int defaultValue)
{
	if ( IsBackTester() ) return defaultValue;

	// Using MT5's built-in
	int earliestBar = iBarShift(symbol, period, earliestTime, false);

	// PrintFormat("earliestBar [%d]", earliestBar);
	return earliestBar;
}
